@@ -8,17 +8,17 @@ module ActionController #:nodoc:
   #
   # Note: To turn off all caching and sweeping, set Base.perform_caching = false.
   module Caching
-    def self.append_features(base) #:nodoc:
-      super
+    def self.included(base) #:nodoc:
       base.send(:include, Pages, Actions, Fragments, Sweeping)
+
       base.class_eval do
         @@perform_caching = true
         cattr_accessor :perform_caching
       end
     end
 
-    # Page caching is an approach to caching where the entire action output of is stored as a HTML file that the web server 
-    # can serve without going through the Action Pack. This can be as much as 100 times faster than going the process of dynamically
+    # Page caching is an approach to caching where the entire action output of is stored as a HTML file that the web server
+    # can serve without going through the Action Pack. This can be as much as 100 times faster than going through the process of dynamically
     # generating the content. Unfortunately, this incredible speed-up is only available to stateless pages where all visitors
     # are treated the same. Content management systems -- including weblogs and wikis -- have many pages that are a great fit
     # for this approach, but account-based systems where people log in and manipulate their own data are often less likely candidates.
@@ -38,9 +38,9 @@ module ActionController #:nodoc:
     #
     #   class WeblogController < ActionController::Base
     #     def update
-    #       List.update(@params["list"]["id"], @params["list"])
-    #       expire_page :action => "show", :id => @params["list"]["id"]
-    #       redirect_to :action => "show", :id => @params["list"]["id"]
+    #       List.update(params[:list][:id], params[:list])
+    #       expire_page :action => "show", :id => params[:list][:id]
+    #       redirect_to :action => "show", :id => params[:list][:id]
     #     end
     #   end
     #
@@ -57,8 +57,7 @@ module ActionController #:nodoc:
     # By default, the cache extension is .html, which makes it easy for the cached files to be picked up by the web server. If you want
     # something else, like .php or .shtml, just set Base.page_cache_extension.
     module Pages
-      def self.append_features(base) #:nodoc:
-        super
+      def self.included(base) #:nodoc:
         base.extend(ClassMethods)
         base.class_eval do
           @@page_cache_directory = defined?(RAILS_ROOT) ? "#{RAILS_ROOT}/public" : ""
@@ -74,35 +73,39 @@ module ActionController #:nodoc:
         #   expire_page "/lists/show"
         def expire_page(path)
           return unless perform_caching
-          File.delete(page_cache_path(path)) if File.exists?(page_cache_path(path))
-          logger.info "Expired page: #{page_cache_file(path)}" unless logger.nil?
+
+          benchmark "Expired page: #{page_cache_file(path)}" do
+            File.delete(page_cache_path(path)) if File.exists?(page_cache_path(path))
+          end
         end
-        
+
         # Manually cache the +content+ in the key determined by +path+. Example:
         #   cache_page "I'm the cached content", "/lists/show"
         def cache_page(content, path)
           return unless perform_caching
-          FileUtils.makedirs(File.dirname(page_cache_path(path)))
-          File.open(page_cache_path(path), "w+") { |f| f.write(content) }
-          logger.info "Cached page: #{page_cache_file(path)}" unless logger.nil?
+
+          benchmark "Cached page: #{page_cache_file(path)}" do
+            FileUtils.makedirs(File.dirname(page_cache_path(path)))
+            File.open(page_cache_path(path), "wb+") { |f| f.write(content) }
+          end
         end
 
         # Caches the +actions+ using the page-caching approach that'll store the cache in a path within the page_cache_directory that
         # matches the triggering url.
         def caches_page(*actions)
           return unless perform_caching
-          actions.each do |action| 
+          actions.each do |action|
             class_eval "after_filter { |c| c.cache_page if c.action_name == '#{action}' }"
           end
         end
-        
+
         private
           def page_cache_file(path)
-            name = ((path.empty? || path == "/") ? "/index" : path)
+            name = ((path.empty? || path == "/") ? "/index" : URI.unescape(path))
             name << page_cache_extension unless (name.split('/').last || name).include? '.'
             return name
           end
-        
+
           def page_cache_path(path)
             page_cache_directory + page_cache_file(path)
           end
@@ -131,13 +134,13 @@ module ActionController #:nodoc:
 
       private
         def caching_allowed
-          !@request.post?
+          !@request.post? && @response.headers['Status'] && @response.headers['Status'].to_i < 400
         end
     end
 
-    # Action caching is similar to page caching by the fact that the entire output of the response is cached, but unlike page caching, 
+    # Action caching is similar to page caching by the fact that the entire output of the response is cached, but unlike page caching,
     # every request still goes through the Action Pack. The key benefit of this is that filters are run before the cache is served, which
-    # allows for authentication and other restrictions on whether someone are supposed to see the cache. Example:
+    # allows for authentication and other restrictions on whether someone is allowed to see the cache. Example:
     #
     #   class ListsController < ApplicationController
     #     before_filter :authenticate, :except => :public
@@ -181,7 +184,7 @@ module ActionController #:nodoc:
         def initialize(*actions)
           @actions = actions
         end
-        
+
         def before(controller)
           return unless @actions.include?(controller.action_name.intern)
           if cache = controller.read_fragment(controller.url_for.split("://").last)
@@ -190,7 +193,7 @@ module ActionController #:nodoc:
             false
           end
         end
-        
+
         def after(controller)
           return if !@actions.include?(controller.action_name.intern) || controller.rendered_action_cache
           controller.write_fragment(controller.url_for.split("://").last, controller.response.body)
@@ -208,7 +211,7 @@ module ActionController #:nodoc:
     #     <%= render_collection_of_partials "topic", Topic.find_all %>
     #   <% end %>
     #
-    # This cache will bind to the name of action that called it. So you would be able to invalidate it using 
+    # This cache will bind to the name of action that called it. So you would be able to invalidate it using
     # <tt>expire_fragment(:controller => "topics", :action => "list")</tt> -- if that was the controller/action used. This is not too helpful
     # if you need to cache multiple fragments per action or if the action itself is cached using <tt>caches_action</tt>. So instead we should
     # qualify the name of the action used with something like:
@@ -225,34 +228,40 @@ module ActionController #:nodoc:
     # In order to use the fragment caching, you need to designate where the caches should be stored. This is done by assigning a fragment store
     # of which there are four different kinds:
     #
-    # * FileStore: Keeps the fragments on disk in the +cache_path+, which works well for all types of environments and share the fragments for
+    # * FileStore: Keeps the fragments on disk in the +cache_path+, which works well for all types of environments and shares the fragments for
     #   all the web server processes running off the same application directory.
     # * MemoryStore: Keeps the fragments in memory, which is fine for WEBrick and for FCGI (if you don't care that each FCGI process holds its
     #   own fragment store). It's not suitable for CGI as the process is thrown away at the end of each request. It can potentially also take
     #   up a lot of memory since each process keeps all the caches in memory.
     # * DRbStore: Keeps the fragments in the memory of a separate, shared DRb process. This works for all environments and only keeps one cache
     #   around for all processes, but requires that you run and manage a separate DRb process.
-    # * MemCachedStore: Works like DRbStore, but uses Danga's MemCached instead.
+    # * MemCacheStore: Works like DRbStore, but uses Danga's MemCache instead.
+    #   Requires the ruby-memcache library:  gem install ruby-memcache.
     #
     # Configuration examples (MemoryStore is the default):
     #
-    #   ActionController::Base.fragment_cache_store = 
-    #     ActionController::Caching::Fragments::MemoryStore.new
-    #
-    #   ActionController::Base.fragment_cache_store = 
-    #     ActionController::Caching::Fragments::FileStore.new("/path/to/cache/directory")
-    #
-    #   ActionController::Base.fragment_cache_store = 
-    #     ActionController::Caching::Fragments::DRbStore.new("druby://localhost:9192")
-    #
-    #   ActionController::Base.fragment_cache_store = 
-    #     ActionController::Caching::Fragments::FileStore.new("localhost")
+    #   ActionController::Base.fragment_cache_store = :memory_store
+    #   ActionController::Base.fragment_cache_store = :file_store, "/path/to/cache/directory"
+    #   ActionController::Base.fragment_cache_store = :drb_store, "druby://localhost:9192"
+    #   ActionController::Base.fragment_cache_store = :mem_cache_store, "localhost"
+    #   ActionController::Base.fragment_cache_store = MyOwnStore.new("parameter")
     module Fragments
       def self.append_features(base) #:nodoc:
         super
         base.class_eval do
           @@fragment_cache_store = MemoryStore.new
-          cattr_accessor :fragment_cache_store
+          cattr_reader :fragment_cache_store
+
+          def self.fragment_cache_store=(store_option)
+            store, *parameters = *([ store_option ].flatten)
+            @@fragment_cache_store = if store.is_a?(Symbol)
+              store_class_name = (store == :drb_store ? "DRbStore" : store.to_s.camelize)
+              store_class = ActionController::Caching::Fragments.const_get(store_class_name)
+              store_class.new(*parameters)
+            else
+              store
+            end
+          end
         end
       end
 
@@ -261,9 +270,9 @@ module ActionController #:nodoc:
       end
 
       # Called by CacheHelper#cache
-      def cache_erb_fragment(block, name = {}, options = {})
+      def cache_erb_fragment(block, name = {}, options = nil)
         unless perform_caching then block.call; return end
-        
+
         buffer = eval("_erbout", block.binding)
 
         if cache = read_fragment(name, options)
@@ -274,124 +283,165 @@ module ActionController #:nodoc:
           write_fragment(name, buffer[pos..-1], options)
         end
       end
-      
-      def write_fragment(name, content, options = {})
+
+      def write_fragment(name, content, options = nil)
         return unless perform_caching
 
         key = fragment_cache_key(name)
-        fragment_cache_store.write(key, content, options)
-        logger.info "Cached fragment: #{key}" unless logger.nil?
+        self.class.benchmark "Cached fragment: #{key}" do
+          fragment_cache_store.write(key, content, options)
+        end
         content
       end
-      
-      def read_fragment(name, options = {})
+
+      def read_fragment(name, options = nil)
         return unless perform_caching
 
         key = fragment_cache_key(name)
-        if cache = fragment_cache_store.read(key, options)
-          logger.info "Fragment hit: #{key}" unless logger.nil?
-          cache
-        else
-          false
+        self.class.benchmark "Fragment read: #{key}" do
+          fragment_cache_store.read(key, options)
         end
       end
-      
+
       # Name can take one of three forms:
       # * String: This would normally take the form of a path like "pages/45/notes"
       # * Hash: Is treated as an implicit call to url_for, like { :controller => "pages", :action => "notes", :id => 45 }
-      # * Regexp: Will destroy all the matched fragments, example: %r{pages/\d*/notes}
-      def expire_fragment(name, options = {})
+      # * Regexp: Will destroy all the matched fragments, example: %r{pages/\d*/notes} Ensure you do not specify start and finish in the regex (^$) because the actual filename matched looks like ./cache/filename/path.cache
+      def expire_fragment(name, options = nil)
         return unless perform_caching
 
         key = fragment_cache_key(name)
 
         if key.is_a?(Regexp)
-          fragment_cache_store.delete_matched(key, options)
-          logger.info "Expired fragments matching: #{key.source}" unless logger.nil?
+          self.class.benchmark "Expired fragments matching: #{key.source}" do
+            fragment_cache_store.delete_matched(key, options)
+          end
         else
-          fragment_cache_store.delete(key, options)
-          logger.info "Expired fragment: #{key}" unless logger.nil?
+          self.class.benchmark "Expired fragment: #{key}" do
+            fragment_cache_store.delete(key, options)
+          end
         end
       end
 
       # Deprecated -- just call expire_fragment with a regular expression
-      def expire_matched_fragments(matcher = /.*/, options = {}) #:nodoc:
+      def expire_matched_fragments(matcher = /.*/, options = nil) #:nodoc:
         expire_fragment(matcher, options)
       end
 
-      class MemoryStore #:nodoc:
-        def initialize
-          @data, @mutex = { }, Mutex.new
+
+      class UnthreadedMemoryStore #:nodoc:
+        def initialize #:nodoc:
+          @data = {}
         end
 
-        def read(name, options = {}) #:nodoc:
-          @mutex.synchronize { @data[name] } rescue nil
+        def read(name, options=nil) #:nodoc:
+          @data[name]
         end
 
-        def write(name, value, options = {}) #:nodoc:
-          @mutex.synchronize { @data[name] = value }
+        def write(name, value, options=nil) #:nodoc:
+          @data[name] = value
         end
 
-        def delete(name, options = {}) #:nodoc:
-          @mutex.synchronize { @data.delete(name) }
+        def delete(name, options=nil) #:nodoc:
+          @data.delete(name)
         end
 
-        def delete_matched(matcher, options) #:nodoc:
-          @mutex.synchronize { @data.delete_if { |k,v| k =~ matcher } }
+        def delete_matched(matcher, options=nil) #:nodoc:
+          @data.delete_if { |k,v| k =~ matcher }
+        end
+      end
+
+      module ThreadSafety #:nodoc:
+        def read(name, options=nil) #:nodoc:
+          @mutex.synchronize { super }
+        end
+
+        def write(name, value, options=nil) #:nodoc:
+          @mutex.synchronize { super }
+        end
+
+        def delete(name, options=nil) #:nodoc:
+          @mutex.synchronize { super }
+        end
+
+        def delete_matched(matcher, options=nil) #:nodoc:
+          @mutex.synchronize { super }
+        end
+      end
+
+      class MemoryStore < UnthreadedMemoryStore #:nodoc:
+        def initialize #:nodoc:
+          super
+          if ActionController::Base.allow_concurrency
+            @mutex = Mutex.new
+            MemoryStore.send(:include, ThreadSafety)
+          end
         end
       end
 
       class DRbStore < MemoryStore #:nodoc:
+        attr_reader :address
+
         def initialize(address = 'druby://localhost:9192')
-          @data, @mutex = DRbObject.new(nil, address), Mutex.new
-        end    
+          super()
+          @address = address
+          @data = DRbObject.new(nil, address)
+        end
       end
 
       class MemCacheStore < MemoryStore #:nodoc:
-        def initialize(address = 'localhost')
-          @data, @mutex = MemCache.new(address), Mutex.new
-        end    
+        attr_reader :addresses
+
+        def initialize(*addresses)
+          super()
+          addresses = addresses.flatten
+          addresses = ["localhost"] if addresses.empty?
+          @addresses = addresses
+          @data = MemCache.new(*addresses)
+        end
       end
 
-      class FileStore #:nodoc:
+      class UnthreadedFileStore #:nodoc:
+        attr_reader :cache_path
+
         def initialize(cache_path)
           @cache_path = cache_path
         end
-    
-        def write(name, value, options = {}) #:nodoc:
+
+        def write(name, value, options = nil) #:nodoc:
           ensure_cache_path(File.dirname(real_file_path(name)))
-          File.open(real_file_path(name), "w+") { |f| f.write(value) }
+          File.open(real_file_path(name), "wb+") { |f| f.write(value) }
         rescue => e
-          Base.logger.info "Couldn't create cache directory: #{name} (#{e.message})" unless Base.logger.nil?
+          Base.logger.error "Couldn't create cache directory: #{name} (#{e.message})" if Base.logger
         end
 
-        def read(name, options = {}) #:nodoc:
-          IO.read(real_file_path(name)) rescue nil
+        def read(name, options = nil) #:nodoc:
+          File.open(real_file_path(name), 'rb') { |f| f.read } rescue nil
         end
 
         def delete(name, options) #:nodoc:
           File.delete(real_file_path(name))
         rescue SystemCallError => e
-          Base.logger.info "Couldn't expire cache #{name} (#{e.message})" unless Base.logger.nil?
+          # If there's no cache, then there's nothing to complain about
         end
 
         def delete_matched(matcher, options) #:nodoc:
           search_dir(@cache_path) do |f|
             if f =~ matcher
-              begin 
+              begin
                 File.delete(f)
               rescue Object => e
-                Base.logger.info "Couldn't expire cache: #{f} (#{e.message})" unless Base.logger.nil?
+                # If there's no cache, then there's nothing to complain about
               end
             end
           end
         end
-    
+
         private
           def real_file_path(name)
             '%s/%s.cache' % [@cache_path, name.gsub('?', '.').gsub(':', '.')]
           end
-        
+
           def ensure_cache_path(path)
             FileUtils.makedirs(path) unless File.exists?(path)
           end
@@ -407,15 +457,25 @@ module ActionController #:nodoc:
               end
             end
           end
-      end
+        end
+
+        class FileStore < UnthreadedFileStore #:nodoc:
+          def initialize(cache_path)
+            super(cache_path)
+            if ActionController::Base.allow_concurrency
+              @mutex = Mutex.new
+              FileStore.send(:include, ThreadSafety)
+            end
+          end
+        end
     end
 
     # Sweepers are the terminators of the caching world and responsible for expiring caches when model objects change.
     # They do this by being half-observers, half-filters and implementing callbacks for both roles. A Sweeper example:
-    # 
+    #
     #   class ListSweeper < ActionController::Caching::Sweeper
     #     observe List, Item
-    #   
+    #
     #     def after_save(record)
     #       list = record.is_a?(List) ? record : record.list
     #       expire_page(:controller => "lists", :action => %w( show public feed ), :id => list.id)
@@ -424,14 +484,14 @@ module ActionController #:nodoc:
     #     end
     #   end
     #
-    # The sweeper is assigned on the controllers that wish to have its job performed using the <tt>cache_sweeper</tt> class method:
+    # The sweeper is assigned in the controllers that wish to have its job performed using the <tt>cache_sweeper</tt> class method:
     #
     #   class ListsController < ApplicationController
     #     caches_action :index, :show, :public, :feed
     #     cache_sweeper :list_sweeper, :only => [ :edit, :destroy, :share ]
     #   end
     #
-    # In the example above, four actions are cached and three actions are responsible of expiring those caches.
+    # In the example above, four actions are cached and three actions are responsible for expiring those caches.
     module Sweeping
       def self.append_features(base) #:nodoc:
         super
@@ -456,10 +516,14 @@ module ActionController #:nodoc:
         end
       end
     end
-    
-    if defined?(ActiveRecord::Observer)
+
+    if defined?(ActiveRecord) and defined?(ActiveRecord::Observer)
       class Sweeper < ActiveRecord::Observer #:nodoc:
         attr_accessor :controller
+
+        # ActiveRecord::Observer will mark this class as reloadable even though it should not be.
+        # However, subclasses of ActionController::Caching::Sweeper should be Reloadable
+        include Reloadable::Subclasses
         
         def before(controller)
           self.controller = controller
@@ -468,17 +532,19 @@ module ActionController #:nodoc:
 
         def after(controller)
           callback(:after)
+          # Clean up, so that the controller can be collected after this request
+          self.controller = nil
         end
-        
+
         private
           def callback(timing)
             controller_callback_method_name = "#{timing}_#{controller.controller_name.underscore}"
             action_callback_method_name     = "#{controller_callback_method_name}_#{controller.action_name}"
-            
+
             send(controller_callback_method_name) if respond_to?(controller_callback_method_name)
             send(action_callback_method_name)     if respond_to?(action_callback_method_name)
           end
-        
+
           def method_missing(method, *arguments)
             return if @controller.nil?
             @controller.send(method, *arguments)
