@@ -11,29 +11,36 @@ require 'active_record/connection_adapters/abstract_adapter'
 # Modifications (ODBC): Mark Imbriaco <mark.imbriaco@pobox.com>
 # Date: 6/26/2005
 #
+# Current maintainer: Ryan Tomayko <rtomayko@gmail.com>
+#
+# Modifications (Migrations): Tom Ward <tom@popdog.net>
+# Date: 27/10/2005
+#
+
 module ActiveRecord
   class Base
     def self.sqlserver_connection(config) #:nodoc:
       require_library_or_gem 'dbi' unless self.class.const_defined?(:DBI)
       
-      symbolize_strings_in_hash(config)
+      config = config.symbolize_keys
 
       mode        = config[:mode] ? config[:mode].to_s.upcase : 'ADO'
       username    = config[:username] ? config[:username].to_s : 'sa'
       password    = config[:password] ? config[:password].to_s : ''
+      autocommit  = config.key?(:autocommit) ? config[:autocommit] : true
       if mode == "ODBC"
         raise ArgumentError, "Missing DSN. Argument ':dsn' must be set in order for this adapter to work." unless config.has_key?(:dsn)
         dsn       = config[:dsn]
-        conn      = DBI.connect("DBI:ODBC:#{dsn}", username, password)
+        driver_url = "DBI:ODBC:#{dsn}"
       else
         raise ArgumentError, "Missing Database. Argument ':database' must be set in order for this adapter to work." unless config.has_key?(:database)
         database  = config[:database]
         host      = config[:host] ? config[:host].to_s : 'localhost'
-        conn      = DBI.connect("DBI:ADO:Provider=SQLOLEDB;Data Source=#{host};Initial Catalog=#{database};User Id=#{username};Password=#{password};")
+        driver_url = "DBI:ADO:Provider=SQLOLEDB;Data Source=#{host};Initial Catalog=#{database};User Id=#{username};Password=#{password};"
       end
-
-      conn["AutoCommit"] = true
-      ConnectionAdapters::SQLServerAdapter.new(conn, logger)
+      conn      = DBI.connect(driver_url, username, password)
+      conn["AutoCommit"] = autocommit
+      ConnectionAdapters::SQLServerAdapter.new(conn, logger, [driver_url, username, password])
     end
   end # class Base
 
@@ -41,11 +48,13 @@ module ActiveRecord
     class ColumnWithIdentity < Column# :nodoc:
       attr_reader :identity, :is_special, :scale
 
-      def initialize(name, default, sql_type = nil, is_identity = false, scale_value = 0)
-        super(name, default, sql_type)
+      def initialize(name, default, sql_type = nil, is_identity = false, null = true, scale_value = 0)
+        super(name, default, sql_type, null)
         @identity = is_identity
         @is_special = sql_type =~ /text|ntext|image/i ? true : false
         @scale = scale_value
+        # SQL Server only supports limits on *char and float types
+        @limit = nil unless @type == :float or @type == :string
       end
 
       def simplified_type(field_type)
@@ -59,6 +68,7 @@ module ActiveRecord
           when /binary|image|varbinary/i                             then :binary
           when /char|nchar|nvarchar|string|varchar/i                 then :string
           when /bit/i                                                then :boolean
+          when /uniqueidentifier/i                                   then :string
         end
       end
 
@@ -71,6 +81,8 @@ module ActiveRecord
         when :datetime  then cast_to_datetime(value)
         when :timestamp then cast_to_time(value)
         when :time      then cast_to_time(value)
+        when :date      then cast_to_datetime(value)
+        when :boolean   then value == true or (value =~ /^t(rue)?$/i) == 0 or value.to_s == '1'
         else value
         end
       end
@@ -98,7 +110,7 @@ module ActiveRecord
 
       # These methods will only allow the adapter to insert binary data with a length of 7K or less
       # because of a SQL Server statement length policy.
-      def string_to_binary(value)
+      def self.string_to_binary(value)
         value.gsub(/(\r|\n|\0|\x1a)/) do
           case $1
             when "\r"   then  "%00"
@@ -109,7 +121,7 @@ module ActiveRecord
         end
       end
 
-      def binary_to_string(value)
+      def self.binary_to_string(value)
         value.gsub(/(%00|%01|%02|%03)/) do
           case $1
             when "%00"    then  "\r"
@@ -162,49 +174,93 @@ module ActiveRecord
     # unixODBC 2.2.11, Ruby ODBC 0.996, Ruby DBI 0.0.23 and Ruby 1.8.2.
     # [Linux strongmad 2.6.11-1.1369_FC4 #1 Thu Jun 2 22:55:56 EDT 2005 i686 i686 i386 GNU/Linux]
     class SQLServerAdapter < AbstractAdapter
+    
+      def initialize(connection, logger, connection_options=nil)
+        super(connection, logger)
+        @connection_options = connection_options
+      end
+
       def native_database_types
         {
           :primary_key => "int NOT NULL IDENTITY(1, 1) PRIMARY KEY",
-          :string      => { :name => "varchar(255)" },
-          :text        => { :name => "text(16)" },
-          :integer     => { :name => "int(4)", :limit => 11 },
-          :float       => { :name => "float(8)" },
-          :datetime    => { :name => "datetime(8)" },
-          :timestamp   => { :name => "datetime(8)" },
-          :time        => { :name => "datetime(8)" },
-          :date        => { :name => "datetime(8)" },
-          :binary      => { :name => "image(16)" },
-          :boolean     => { :name => "bit(1)" }
+          :string      => { :name => "varchar", :limit => 255  },
+          :text        => { :name => "text" },
+          :integer     => { :name => "int" },
+          :float       => { :name => "float", :limit => 8 },
+          :datetime    => { :name => "datetime" },
+          :timestamp   => { :name => "datetime" },
+          :time        => { :name => "datetime" },
+          :date        => { :name => "datetime" },
+          :binary      => { :name => "image"},
+          :boolean     => { :name => "bit"}
         }
       end
 
       def adapter_name
         'SQLServer'
       end
+      
+      def supports_migrations? #:nodoc:
+        true
+      end
+
+      # CONNECTION MANAGEMENT ====================================#
+
+      # Returns true if the connection is active.
+      def active?
+        @connection.execute("SELECT 1") { }
+        true
+      rescue DBI::DatabaseError, DBI::InterfaceError
+        false
+      end
+
+      # Reconnects to the database, returns false if no connection could be made.
+      def reconnect!
+        disconnect!
+        @connection = DBI.connect(*@connection_options)
+      rescue DBI::DatabaseError => e
+        @logger.warn "#{adapter_name} reconnection failed: #{e.message}" if @logger
+        false
+      end
+      
+      # Disconnects from the database
+      
+      def disconnect!
+        @connection.disconnect rescue nil
+      end
 
       def select_all(sql, name = nil)
         select(sql, name)
       end
 
       def select_one(sql, name = nil)
-        add_limit!(sql, nil)
+        add_limit!(sql, :limit => 1)
         result = select(sql, name)
         result.nil? ? nil : result.first
       end
 
       def columns(table_name, name = nil)
-        sql = "SELECT COLUMN_NAME as ColName, COLUMN_DEFAULT as DefaultValue, DATA_TYPE as ColType, COL_LENGTH('#{table_name}', COLUMN_NAME) as Length, COLUMNPROPERTY(OBJECT_ID('#{table_name}'), COLUMN_NAME, 'IsIdentity') as IsIdentity, NUMERIC_SCALE as Scale FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '#{table_name}'"
+        return [] if table_name.blank?
+        table_name = table_name.to_s if table_name.is_a?(Symbol)
+        table_name = table_name.split('.')[-1] unless table_name.nil?
+        sql = "SELECT COLUMN_NAME as ColName, COLUMN_DEFAULT as DefaultValue, DATA_TYPE as ColType, IS_NULLABLE As IsNullable, COL_LENGTH('#{table_name}', COLUMN_NAME) as Length, COLUMNPROPERTY(OBJECT_ID('#{table_name}'), COLUMN_NAME, 'IsIdentity') as IsIdentity, NUMERIC_SCALE as Scale FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '#{table_name}'"
         # Comment out if you want to have the Columns select statment logged.
-        # Personnally, I think it adds unneccessary bloat to the log. 
+        # Personally, I think it adds unnecessary bloat to the log. 
         # If you do comment it out, make sure to un-comment the "result" line that follows
         result = log(sql, name) { @connection.select_all(sql) }
         #result = @connection.select_all(sql)
         columns = []
-        result.each { |field| columns << ColumnWithIdentity.new(field[:ColName], field[:DefaultValue].to_s.gsub!(/[()\']/,"") =~ /null/ ? nil : field[:DefaultValue], "#{field[:ColType]}(#{field[:Length]})", field[:IsIdentity] == 1 ? true : false, field[:Scale]) }
+        result.each do |field|
+          default = field[:DefaultValue].to_s.gsub!(/[()\']/,"") =~ /null/ ? nil : field[:DefaultValue]
+          type = "#{field[:ColType]}(#{field[:Length]})"
+          is_identity = field[:IsIdentity] == 1
+          is_nullable = field[:IsNullable] == 'YES'
+          columns << ColumnWithIdentity.new(field[:ColName], default, type, is_identity, is_nullable, field[:Scale])
+        end
         columns
       end
 
-      def insert(sql, name = nil, pk = nil, id_value = nil)
+      def insert(sql, name = nil, pk = nil, id_value = nil, sequence_name = nil)
         begin
           table_name = get_table_name(sql)
           col = get_identity_column(table_name)
@@ -222,7 +278,7 @@ module ActiveRecord
           end
           log(sql, name) do
             @connection.execute(sql)
-            select_one("SELECT @@IDENTITY AS Ident")["Ident"]
+            id_value || select_one("SELECT @@IDENTITY AS Ident")["Ident"]
           end
         ensure
           if ii_enabled
@@ -274,8 +330,8 @@ module ActiveRecord
       def quote(value, column = nil)
         case value
           when String                
-            if column && column.type == :binary
-              "'#{quote_string(column.string_to_binary(value))}'"
+            if column && column.type == :binary && column.class.respond_to?(:string_to_binary)
+              "'#{quote_string(column.class.string_to_binary(value))}'"
             else
               "'#{quote_string(value)}'"
             end
@@ -293,20 +349,47 @@ module ActiveRecord
         string.gsub(/\'/, "''")
       end
 
+      def quoted_true
+        "1"
+      end
+      
+      def quoted_false
+        "0"
+      end
+
       def quote_column_name(name)
         "[#{name}]"
       end
 
       def add_limit_offset!(sql, options)
-        if options.has_key?(:limit) and options.has_key?(:offset) and !options[:limit].nil? and !options[:offset].nil?
-          options[:order] ||= "id ASC"
-          total_rows = @connection.select_all("SELECT count(*) as TotalRows from #{get_table_name(sql)}")[0][:TotalRows].to_i
-          if (options[:limit] + options[:offset]) > total_rows
-            options[:limit] = (total_rows - options[:offset] > 0) ? (total_rows - options[:offset]) : 1
+        if options[:limit] and options[:offset]
+          total_rows = @connection.select_all("SELECT count(*) as TotalRows from (#{sql.gsub(/\bSELECT\b/i, "SELECT TOP 1000000000")}) tally")[0][:TotalRows].to_i
+          if (options[:limit] + options[:offset]) >= total_rows
+            options[:limit] = (total_rows - options[:offset] >= 0) ? (total_rows - options[:offset]) : 0
           end
-          sql.gsub!(/SELECT/i, "SELECT * FROM ( SELECT TOP #{options[:limit]} * FROM ( SELECT TOP #{options[:limit] + options[:offset]}")<<" ) AS tmp1 ORDER BY #{change_order_direction(options[:order])} ) AS tmp2 ORDER BY #{options[:order]}"
-        else
-          sql.gsub!(/SELECT/i, "SELECT TOP #{options[:limit]}") unless options[:limit].nil?
+          sql.sub!(/^\s*SELECT/i, "SELECT * FROM (SELECT TOP #{options[:limit]} * FROM (SELECT TOP #{options[:limit] + options[:offset]} ")
+          sql << ") AS tmp1"
+          if options[:order]
+            options[:order] = options[:order].split(',').map do |field|
+              parts = field.split(" ")
+              tc = parts[0]
+              if sql =~ /\.\[/ and tc =~ /\./ # if column quoting used in query
+                tc.gsub!(/\./, '\\.\\[')
+                tc << '\\]'
+              end
+              if sql =~ /#{tc} AS (t\d_r\d\d?)/
+                  parts[0] = $1
+              end
+              parts.join(' ')
+            end.join(', ')
+            sql << " ORDER BY #{change_order_direction(options[:order])}) AS tmp2 ORDER BY #{options[:order]}"
+          else
+            sql << " ) AS tmp2"
+          end
+        elsif sql !~ /^\s*SELECT (@@|COUNT\()/i
+          sql.sub!(/^\s*SELECT([\s]*distinct)?/i) do
+            "SELECT#{$1} TOP #{options[:limit]}"
+          end unless options[:limit].nil?
         end
       end
 
@@ -322,6 +405,78 @@ module ActiveRecord
       def create_database(name)
         execute "CREATE DATABASE #{name}"
       end
+   
+      def current_database
+        @connection.select_one("select DB_NAME()")[0]
+      end
+
+      def tables(name = nil)
+        execute("SELECT table_name from information_schema.tables WHERE table_type = 'BASE TABLE'", name).inject([]) do |tables, field|
+          table_name = field[0]
+          tables << table_name unless table_name == 'dtproperties'
+          tables
+        end
+      end
+
+      def indexes(table_name, name = nil)
+        indexes = []
+        execute("EXEC sp_helpindex #{table_name}", name).each do |index| 
+          unique = index[1] =~ /unique/
+          primary = index[1] =~ /primary key/
+          if !primary
+            indexes << IndexDefinition.new(table_name, index[0], unique, index[2].split(", "))
+          end
+        end
+        indexes
+      end
+            
+      def rename_table(name, new_name)
+        execute "EXEC sp_rename '#{name}', '#{new_name}'"
+      end
+      
+      def remove_column(table_name, column_name)
+        execute "ALTER TABLE #{table_name} DROP COLUMN #{column_name}"
+      end 
+      
+      def rename_column(table, column, new_column_name)
+        execute "EXEC sp_rename '#{table}.#{column}', '#{new_column_name}'"
+      end
+      
+      def change_column(table_name, column_name, type, options = {}) #:nodoc:
+        sql_commands = ["ALTER TABLE #{table_name} ALTER COLUMN #{column_name} #{type_to_sql(type, options[:limit])}"]
+        if options[:default]
+          remove_default_constraint(table_name, column_name)
+          sql_commands << "ALTER TABLE #{table_name} ADD CONSTRAINT DF_#{table_name}_#{column_name} DEFAULT #{options[:default]} FOR #{column_name}"
+        end
+        sql_commands.each {|c|
+          execute(c)
+        }
+      end
+      
+      def remove_column(table_name, column_name)
+        remove_default_constraint(table_name, column_name)
+        execute "ALTER TABLE #{table_name} DROP COLUMN #{column_name}"
+      end
+      
+      def remove_default_constraint(table_name, column_name)
+        defaults = select "select def.name from sysobjects def, syscolumns col, sysobjects tab where col.cdefault = def.id and col.name = '#{column_name}' and tab.name = '#{table_name}' and col.id = tab.id"
+        defaults.each {|constraint|
+          execute "ALTER TABLE #{table_name} DROP CONSTRAINT #{constraint["name"]}"
+        }
+      end
+      
+      def remove_index(table_name, options = {})
+        execute "DROP INDEX #{table_name}.#{index_name(table_name, options)}"
+      end
+
+      def type_to_sql(type, limit = nil) #:nodoc:
+        native = native_database_types[type]
+        # if there's no :limit in the default type definition, assume that type doesn't support limits
+        limit = limit || native[:limit]
+        column_type_sql = native[:name]
+        column_type_sql << "(#{limit})" if limit
+        column_type_sql
+      end
 
       private
         def select(sql, name = nil)
@@ -332,6 +487,7 @@ module ActiveRecord
               record = {}
               row.column_names.each do |col|
                 record[col] = row[col]
+                record[col] = record[col].to_time if record[col].is_a? DBI::Timestamp
               end
               rows << record
             end
@@ -346,9 +502,9 @@ module ActiveRecord
         end
 
         def get_table_name(sql)
-          if sql =~ /into\s*([^\s]+)\s*|update\s*([^\s]+)\s*/i
+          if sql =~ /^\s*insert\s+into\s+([^\(\s]+)\s*|^\s*update\s+([^\(\s]+)\s*/i
             $1
-          elsif sql =~ /from\s*([^\s]+)\s*/i
+          elsif sql =~ /from\s+([^\(\s]+)\s*/i
             $1
           else
             nil
@@ -374,17 +530,19 @@ module ActiveRecord
         end
 
         def change_order_direction(order)
-          case order
-            when  /DESC/i     then order.gsub(/DESC/i, "ASC")
-            when  /ASC/i      then order.gsub(/ASC/i, "DESC")
-            else                   String.new(order).insert(-1, " DESC")
-          end
+          order.split(",").collect {|fragment|
+            case fragment
+              when  /\bDESC\b/i     then fragment.gsub(/\bDESC\b/i, "ASC")
+              when  /\bASC\b/i      then fragment.gsub(/\bASC\b/i, "DESC")
+              else                  String.new(fragment).split(',').join(' DESC,') + ' DESC'
+            end
+          }.join(",")
         end
 
         def get_special_columns(table_name)
           special = []
-          @table_columns = {} unless @table_columns
-          @table_columns[table_name] = columns(table_name) if @table_columns[table_name] == nil
+          @table_columns ||= {}
+          @table_columns[table_name] ||= columns(table_name)
           @table_columns[table_name].each do |col|
             special << col.name if col.is_special
           end
@@ -395,6 +553,7 @@ module ActiveRecord
           special_cols = get_special_columns(get_table_name(sql))
           for col in special_cols.to_a
             sql.gsub!(Regexp.new(" #{col.to_s} = "), " #{col.to_s} LIKE ")
+            sql.gsub!(/ORDER BY #{col.to_s}/i, '')
           end
           sql
         end
