@@ -1,6 +1,8 @@
+require 'fixtures/topic'
+
 class Reply < Topic
   belongs_to :topic, :foreign_key => "parent_id", :counter_cache => true
-  has_many :silly_replies, :dependent => true, :foreign_key => "parent_id"
+  has_many :replies, :class_name => "SillyReply", :dependent => :destroy, :foreign_key => "parent_id"
 
   validate :errors_on_empty_content
   validate_on_create :title_is_wrong_create
@@ -31,4 +33,5 @@ class Reply < Topic
 end
 
 class SillyReply < Reply
-end
\ No newline at end of file
+  belongs_to :reply, :foreign_key => "parent_id", :counter_cache => :replies_count
+end
