@@ -1,4 +1,5 @@
 require 'date'
+require 'time'
 
 module ActiveSupport #:nodoc:
   module CoreExtensions #:nodoc:
@@ -6,9 +7,10 @@ module ActiveSupport #:nodoc:
       # Getting times in different convenient string representations and other objects
       module Conversions
         DATE_FORMATS = {
-          :db    => "%Y-%m-%d %H:%M:%S",
-          :short => "%e %b %H:%M",
-          :long  => "%B %e, %Y %H:%M"
+          :db     => "%Y-%m-%d %H:%M:%S",
+          :short  => "%d %b %H:%M",
+          :long   => "%B %d, %Y %H:%M",
+          :rfc822 => "%a, %d %b %Y %H:%M:%S %z"
         }
 
         def self.append_features(klass)
@@ -32,4 +34,4 @@ module ActiveSupport #:nodoc:
       end
     end
   end
-end
\ No newline at end of file
+end
